/**
 * @file draw.h
 * @author Tanguy Cavagna (tanguy.cavagna@etu.hesge.ch)
 * @brief Header of the draw file
 * @version 0.1
 * @date 2022-04-13
 *
 * @copyright Copyright (c) 2021
 *
 */

#ifndef _DRAW_H_
#define _DRAW_H_

#include "../charge/charge.h"
#include "../vec2/vec2.h"
#include <GL/freeglut.h>
#include <GL/gl.h>
#include <float.h>
#include <math.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define CHAR_WIDTH 8
#define CHAR_HEIGHT 13

/**
 * @brief Draw a text
 *
 * @param pos
 * @param text
 */
void draw_textbox(vec2 pos, char *text);

/**
 * @brief Draw a segment of a field line between two given points
 *
 * @param charges All the charges
 * @param num_charges Number of charges
 * @param dx Distance between next point and current point
 * @param w Window width
 * @param h Window height
 * @param p Current pos
 * @param next Next pos
 * @param side Side of the line
 */
void draw_field_line_segment(charge_t *charges, int num_charges, double dx,
                             int w, int h, vec2 p, vec2 next, bool side);

/**
 * @brief Draw a field line
 *
 * @param charges All the charges in action
 * @param num_charges Number of charges
 * @param dx Distance between  current and next point to draw line
 * @param pos0 Line starting point (will draw on both side of the point)
 * @param w Window width
 * @param h Window height
 */
void draw_field_line(charge_t *charges, int num_charges, double dx, vec2 pos0,
                     int w, int h);

/**
 * @brief Draw a heatmap of the intensity
 *
 * @param charges
 * @param num_charges
 * @param w
 * @param h
 */
void draw_heatmap(charge_t *charges, int num_charges, int w, int h);

/**
 * @brief Draw a vector field
 *
 * @param charges
 * @param num_charges
 * @param points
 * @param nb_points
 * @param w
 * @param h
 */
void draw_vector_field(charge_t *charges, int num_charges, vec2 *points,
                       int nb_points, int w, int h);

/**
 * @brief Draw all charges
 *        A circle with a minus sign for negative charges
 *        A circle with a plus sign for positive charges
 *
 * @param charges All the charges
 * @param num_charges Number of charges in action
 * @param w Window width
 * @param h Window height
 */
void draw_charges(charge_t *charges, int num_charges, int w, int h);

#endif
